/* 
groundwater.c
calculate the effect of gound water depth in the soil

*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
Biome-BGCMuSo v6.2.
Copyright 2020, D. Hidy [dori.hidy@gmail.com]
Hungarian Academy of Sciences, Hungary
See the website of Biome-BGCMuSo at http://nimbus.elte.hu/bbgc/ for documentation, model executable and example input files.
*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include "ini.h"
#include "bgc_struct.h"
#include "pointbgc_struct.h"
#include "pointbgc_func.h"
#include "bgc_constants.h"



int groundwater(const control_struct* ctrl, const siteconst_struct* sitec, soilprop_struct* sprop, epvar_struct* epv, wstate_struct* ws, wflux_struct* wf)
{
	int layerSAT,errorCode,layer,layerLOW;
	double soilw_fc,soilw_sat;
	double GWdist_crit, GWdist, GWeff, GWboundL,GWboundU,critVWCdiff;

	layer=layerLOW=layerSAT=errorCode=0;
	GWdist=0;

	/* critical distance from groundwater table from where actual layer is affected */
	GWdist_crit = 2;

	/* critical VWC-difference */
	critVWCdiff = 0.01;

	/* if gound water depth information is available using in multilayer calculation */
	if (ctrl->GWD_flag)	
	{
		/* in spinup and transient run only 365 values are used (daily averages) */
		if (ctrl->spinup == 0)
			sprop->GWD = sprop->GWD_array[(ctrl->simyr * nDAYS_OF_YEAR) + ctrl->yday]; 
		else
			sprop->GWD = sprop->GWD_array[ctrl->yday];
		
		sprop->GWlayer=DATA_GAP;

		/* calculate processes while layer is enough close to groundwater table */
		layer = N_SOILLAYERS-1;
		while (GWdist <= GWdist_crit && layer >= 0)
		{
			soilw_fc     = sprop->VWCfc[layer]   * sitec->soillayer_thickness[layer] * water_density;

			/* if water table (GW) is closer than a critical distance (200 cm), the effect of GW is noticable: 
					higher FC values - less holding capacity (SAT-FC) 
					plus water from groundwater table */


			GWdist   = sprop->GWD - sitec->soillayer_midpoint[layer];
			GWboundL  = sitec->soillayer_depth[layer];
			if (layer == 0)
				GWboundU  = -1*CRIT_PREC;
			else
				GWboundU  = sitec->soillayer_depth[layer-1];

			if (GWdist <= GWdist_crit)
			{
				/* if groundwater table is in actual layer (above lower boundary): GWlayer = actual layer, lower layer is charged */
				if (sprop->GWD <= GWboundL && sprop->GWD > GWboundU) 
				{
					sprop->GWlayer=(double) layer;
					/* soil layers below the groundwater table are saturated - net water gain from soil system */
					for (layerSAT = 1; layer+layerSAT < N_SOILLAYERS; layerSAT++)
					{
						soilw_sat                             = sprop->VWCsat[layer+layerSAT] * sitec->soillayer_thickness[layer+layerSAT] * water_density;
						wf->soilw_from_GW[layer+layerSAT]    += (soilw_sat - ws->soilw[layer+layerSAT]);
						ws->soilw[layer+layerSAT]             = soilw_sat;
						epv->VWC[layer+layerSAT]              = sprop->VWCsat[layer+layerSAT];
						sprop->VWCfc[layer+layerSAT]          = sprop->VWCsat[layer+layerSAT];
					}
				}
				
				/* GWeff is calculated if lower layer contains GW table and it has already charged */
				if (layer == N_SOILLAYERS-1 || 
				   (layer < N_SOILLAYERS-1 && (sprop->VWCfc[layer+1]-epv->VWC[layer+1]) < critVWCdiff && sprop->VWCfc[layer+1] > sprop->VWCfc_base[layer+1]))
				{
					/* if groundwater table is below the midpoint -> GWdist is positive -> GWeff is less than 1 */
					if (GWdist > 0)
						GWeff=1-GWdist/GWdist_crit;
					else
						GWeff=1;

					/* higher FC values */
					sprop->VWCfc[layer] = sprop->VWCfc_base[layer] + GWeff * (sprop->VWCsat[layer]-sprop->VWCfc_base[layer]);
					soilw_fc            = sprop->VWCfc[layer]   * sitec->soillayer_thickness[layer] * water_density;
					
					/* plus water from groundwater table */
					if (epv->VWC[layer] < sprop->VWCfc[layer]) 
					{	
						wf->soilw_from_GW[layer] += (soilw_fc - ws->soilw[layer]) * sprop->drain_coeff[layer];
						ws->soilw[layer]         += wf->soilw_from_GW[layer];
						epv->VWC[layer]           = ws->soilw[layer] / (sitec->soillayer_thickness[layer] * water_density);
					}
				}
			}
			else
			{
				sprop->VWCfc[layer] = sprop->VWCfc_base[layer];
			}
			layer = layer-1;
		}



	
	}
	else
		sprop->GWD = DATA_GAP;



		
	return (errorCode);
}
